@@ -1,6 +1,6 @@
-/* $Id: rmagick.h,v 1.60 2004/08/02 22:39:15 rmagick Exp $ */
+/* $Id: rmagick.h,v 1.105 2006/02/19 17:12:20 rmagick Exp $ */
 /*=============================================================================
-|               Copyright (C) 2004 by Timothy P. Hunter
+|               Copyright (C) 2006 by Timothy P. Hunter
 | Name:     rmagick.h
 | Purpose:  RMagick declarations and definitions
 | Author:   Tim Hunter
@@ -19,6 +19,14 @@
 #include "ruby.h"
 #include "intern.h"
 #include "rubyio.h"
+
+
+// Undef Ruby's versions of these symbols
+#undef PACKAGE_VERSION
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE_TARNAME
 #include "magick/api.h"
 
 
@@ -43,10 +51,15 @@
 #define RUBY16(d) d
 #endif
 
+// Backport these definitions to Ruby 1.6.7
 #if !defined(ULONG2NUM)
 #define ULONG2NUM(v) UINT2NUM(v)
 #endif
 
+#if !defined(LONG2NUM)
+#define LONG2NUM(v) INT2NUM(v)
+#endif
+
 
 // Define a version of StringValuePtr that works in both 1.6 and 1.8.
 #if !defined(StringValuePtr)
@@ -61,6 +74,10 @@
 #undef DegreesToRadians     // defined in ImageMagick.h in 6.0.2
 #define DegreesToRadians(x) ((x)*3.14159265358979323846/180.0)
 
+#define LMAX(a,b) ((((long)(a))>((long)(b)))?((long)(a)):((long)(b)))
+#define FMAX(a,b) ((((double)(a))>((double)(b)))?((double)(a)):((double)(b)))
+#define FMIN(a,b) ((((double)(a))<=((double)(b)))?((double)(a)):((double)(b)))
+
 typedef ImageInfo Info; // Make type name match class name
 typedef PixelPacket Pixel;
 
@@ -72,11 +89,17 @@ typedef struct
 } Montage;
 
 // Draw
+struct TmpFile_Name
+{
+    struct TmpFile_Name *next;
+    char name[1];
+};
+
 typedef struct
 {
     DrawInfo *info;         // the DrawInfo struct
     VALUE primitives;       // the primitive string
-    VALUE tmpfile_ary;
+    struct TmpFile_Name *tmpfile_ary;
 } Draw;             // make the type match the class name
 
 // Enum
@@ -154,28 +177,38 @@ typedef enum _QuantumExpressionOperator
     ImageMagick used simply size_t and off_t in 5.5.1, then defined the
     Extended(Un)SignedIntegralType from 5.5.2 thru 5.5.7. The 5.5.8 release
     deprecates these types and uses Magick(Un)SignedType instead.
-    GraphicsMagick 1.1. introduced the magick_(u)int64_t type.
+    Prior to 1.1, GraphicsMagick defined the Extended(Un)SignedIntegralType(s).
+    GraphicsMagick 1.1. introduced the magick_(u)int64_t types.
 
-    Here, if we don't already have magick_(u)int64_t, define them.
+    Here, if we don't already have magick_(u)int64_t, define them in terms
+    of whatever other types are defined.
 */
 #if !defined(HAVE_MAGICK_INT64_T)
-#if defined(HAVE_MAGICKOFFSETTYPE)
-typedef MagickOffsetType magick_int64_t;
-#elif defined(HAVE_EXTENDEDSIGNEDINTEGRALTYPE)
-typedef ExtendedSignedIntegralType magick_int64_t;
-#else
-typedef off_t magick_int64_t;
-#endif
+    #if defined(HAVE_MAGICKOFFSETTYPE)
+        typedef MagickOffsetType magick_int64_t;
+    #elif defined(HAVE_EXTENDEDSIGNEDINTEGRALTYPE)
+        typedef ExtendedSignedIntegralType magick_int64_t;
+    #else
+        typedef off_t magick_int64_t;
+    #endif
 #endif
 
 #if !defined(HAVE_MAGICK_UINT64_T)
-#if defined(HAVE_MAGICKSIZETYPE)
-typedef MagickSizeType magick_uint64_t;
-#elif defined(HAVE_EXTENDEDUNSIGNEDINTEGRALTYPE)
-typedef ExtendedUnsignedIntegralType magick_uint64_t;
-#else
-typedef size_t magick_uint64_t;
+    #if defined(HAVE_MAGICKSIZETYPE)
+        typedef MagickSizeType magick_uint64_t;
+    #elif defined(HAVE_EXTENDEDUNSIGNEDINTEGRALTYPE)
+        typedef ExtendedUnsignedIntegralType magick_uint64_t;
+    #else
+        typedef size_t magick_uint64_t;
+    #endif
 #endif
+
+#if !defined(HAVE_MAGICKBOOLEANTYPE)
+typedef unsigned int MagickBooleanType;
+#endif
+
+#if !defined(HAVE_UNDEFINEDGRAVITY)
+#define UndefinedGravity 0
 #endif
 
 // This implements the "omitted storage class model" for external variables.
@@ -242,20 +275,25 @@ EXTERN VALUE Class_GravityType;
 EXTERN VALUE Class_ImageType;
 EXTERN VALUE Class_InterlaceType;
 EXTERN VALUE Class_NoiseType;
+#if defined(HAVE_IMAGE_ORIENTATION)
+EXTERN VALUE Class_OrientationType;
+#endif
 EXTERN VALUE Class_PaintMethod;
 EXTERN VALUE Class_PreviewType;
 EXTERN VALUE Class_RenderingIntent;
 EXTERN VALUE Class_ResolutionType;
+EXTERN VALUE Class_StorageType;
 EXTERN VALUE Class_StretchType;
 EXTERN VALUE Class_StyleType;
 EXTERN VALUE Class_WeightType;
+EXTERN VALUE Class_VirtualPixelMethod;
 
 /*
 *   Commonly-used IDs
 */
 EXTERN ID ID__dummy_img_;       // "_dummy_img_"
-EXTERN ID ID_changed;           // "changed"
 EXTERN ID ID_call;              // "call"
+EXTERN ID ID_changed;           // "changed"
 EXTERN ID ID_cur_image;         // "cur_image"
 EXTERN ID ID_dup;               // "dup"
 EXTERN ID ID_enumerators;       // "enumerators"
@@ -278,19 +316,6 @@ EXTERN ID ID_x;                 // "x"
 EXTERN ID ID_y;                 // "y"
 
 
-#if defined(HAVE_GETNEXTIMAGEINLIST)
-#define GET_NEXT_IMAGE(a) GetNextImageInList(a)
-#else
-#define GET_NEXT_IMAGE(a) (a)->next
-#endif
-
-#if defined(HAVE_GETLOCALEEXCEPTIONMESSAGE)
-#define GET_MSG(s,t) GetLocaleExceptionMessage((s), (t))
-#else
-#define GET_MSG(s,t) t
-#endif
-
-
 #define min(a,b) ((a)<(b)?(a):(b))
 #define max(a,b) ((a)>(b)?(a):(b))
 
@@ -306,7 +331,7 @@ EXTERN ID ID_y;                 // "y"
 */
 #define HANDLE_ERROR rm_handle_error(&exception);
 // For ExceptionInfo structures in images.
-#define HANDLE_IMG_ERROR(img) rm_handle_error(&((img)->exception));
+#define HANDLE_ERROR_IMG(img) rm_handle_error(&((img)->exception));
 
 /*
     Map the QuantumDepth to a StorageType.
@@ -386,6 +411,70 @@ EXTERN ID ID_y;                 // "y"
     ATTR_READER(class, attr)\
     ATTR_WRITER(class, attr)
 
+/*
+ *  Define functions to get/set attributes in Image::Info that
+ *  use the Get/SetImageOption API.
+*/
+#define OPTION_ATTR_READER(opt, key) \
+    VALUE Info_##opt(VALUE self)\
+    {\
+        return get_option(self, #key);\
+    }
+#define OPTION_ATTR_WRITER(opt, key) \
+    VALUE Info_##opt##_eq(VALUE self, VALUE string)\
+    {\
+        return set_option(self, #key, string);\
+    }
+#define OPTION_ATTR_ACCESSOR(opt, key)\
+    OPTION_ATTR_READER(opt, key)\
+    OPTION_ATTR_WRITER(opt, key)
+
+
+/*
+ *  Declare Pixel channel attribute writers
+*/
+#define DEF_PIXEL_CHANNEL_WRITER(_channel_) \
+extern VALUE \
+Pixel_##_channel_##_eq(VALUE self, VALUE v) \
+{ \
+    Pixel *pixel; \
+ \
+    rm_check_frozen(self); \
+    Data_Get_Struct(self, Pixel, pixel); \
+    pixel->_channel_ = (Quantum) NUM2UINT(v); \
+    rb_funcall(self, ID_changed, 0); \
+    rb_funcall(self, ID_notify_observers, 1, self); \
+    return INT2NUM(pixel->_channel_); \
+}
+
+
+/*
+ *  Declare Pixel CMYK channel attribute accessors
+*/
+#define DEF_PIXEL_CMYK_CHANNEL_ACCESSOR(_cmyk_channel_, _rgb_channel_) \
+extern VALUE \
+Pixel_##_cmyk_channel_##_eq(VALUE self, VALUE v) \
+{ \
+    Pixel *pixel; \
+ \
+    rm_check_frozen(self); \
+    Data_Get_Struct(self, Pixel, pixel); \
+    pixel->_rgb_channel_ = (Quantum) NUM2UINT(v); \
+    rb_funcall(self, ID_changed, 0); \
+    rb_funcall(self, ID_notify_observers, 1, self); \
+    return INT2NUM(pixel->_rgb_channel_); \
+} \
+ \
+extern VALUE \
+Pixel_##_cmyk_channel_(VALUE self) \
+{ \
+    Pixel *pixel; \
+ \
+    Data_Get_Struct(self, Pixel, pixel); \
+    return INT2NUM(pixel->_rgb_channel_); \
+}
+
+
 /*
  *  Enum constants - define a subclass of Enum for the specified enumeration.
  *  Define an instance of the subclass for each member in the enumeration.
@@ -415,6 +504,7 @@ EXTERN ID ID_y;                 // "y"
    e = (type)(magick_enum->val);\
    } while(0)
 
+#define ENUM_TO_NAME(_enum) case _enum: return #_enum;
 
 
 // Method, external function declarations. These declarations are
@@ -427,6 +517,7 @@ EXTERN ID ID_y;                 // "y"
 // rmdraw.c
 ATTR_WRITER(Draw, affine)
 ATTR_WRITER(Draw, align)
+ATTR_WRITER(Draw, border_color)
 ATTR_WRITER(Draw, decorate)
 ATTR_WRITER(Draw, density)
 ATTR_WRITER(Draw, encoding)
@@ -448,6 +539,7 @@ extern VALUE Draw_clone(VALUE);
 extern VALUE Draw_composite(int, VALUE *, VALUE);
 extern VALUE Draw_draw(VALUE, VALUE);
 extern VALUE Draw_dup(VALUE);
+extern VALUE Draw_get_multiline_type_metrics(int, VALUE *, VALUE);
 extern VALUE Draw_get_type_metrics(int, VALUE *, VALUE);
 extern VALUE Draw_init_copy(VALUE, VALUE);
 extern VALUE Draw_initialize(VALUE);
@@ -506,35 +598,48 @@ extern VALUE ImageList_to_blob(VALUE);
 extern VALUE ImageList_write(VALUE, VALUE);
 
 extern VALUE rm_imagelist_new(void);
+extern VALUE rm_imagelist_from_images(Image *);
+extern Image *rm_images_from_imagelist(VALUE);
+extern VALUE rm_imagelist_scene_eq(VALUE, VALUE);
 extern int rm_imagelist_length(VALUE);
-extern VALUE rm_imagelist_push(VALUE, VALUE);
+extern void rm_imagelist_push(VALUE, VALUE);
 
 
 // rminfo.c
 ATTR_ACCESSOR(Info, antialias)
+ATTR_ACCESSOR(Info, authenticate)
 ATTR_ACCESSOR(Info, background_color)
 ATTR_ACCESSOR(Info, border_color)
 ATTR_ACCESSOR(Info, colorspace)
+ATTR_ACCESSOR(Info, comment)
 ATTR_ACCESSOR(Info, compression)
+ATTR_ACCESSOR(Info, delay)
 ATTR_ACCESSOR(Info, density)
 ATTR_ACCESSOR(Info, depth)
+ATTR_ACCESSOR(Info, dispose)
 ATTR_ACCESSOR(Info, dither)
 ATTR_ACCESSOR(Info, extract)
 ATTR_ACCESSOR(Info, filename)
+ATTR_ACCESSOR(Info, fill)
 ATTR_ACCESSOR(Info, font)
 ATTR_ACCESSOR(Info, format)
 ATTR_ACCESSOR(Info, fuzz)
+ATTR_ACCESSOR(Info, gravity)
 ATTR_ACCESSOR(Info, group)
 ATTR_ACCESSOR(Info, image_type)
 ATTR_ACCESSOR(Info, interlace)
+ATTR_ACCESSOR(Info, label)
 ATTR_ACCESSOR(Info, matte_color)
+ATTR_WRITER(Info, monitor)
 ATTR_ACCESSOR(Info, monochrome)
 ATTR_ACCESSOR(Info, number_scenes)
+ATTR_ACCESSOR(Info, orientation)
 ATTR_ACCESSOR(Info, page)
 ATTR_ACCESSOR(Info, pen)
 // ATTR_ACCESSOR(Info, ping) obsolete
-// ATTR_ACCESSOR(Info, pointsize)
+ATTR_ACCESSOR(Info, pointsize)
 ATTR_ACCESSOR(Info, quality)
+ATTR_ACCESSOR(Info, sampling_factor)
 ATTR_ACCESSOR(Info, scene)
 ATTR_ACCESSOR(Info, server_name)
 ATTR_ACCESSOR(Info, subimage)
@@ -552,6 +657,9 @@ extern VALUE Info_new(VALUE);
 #endif
 
 extern VALUE Info_define(int, VALUE *, VALUE);
+extern VALUE Info_aset(VALUE, VALUE, VALUE, VALUE);
+extern VALUE Info_aref(VALUE, VALUE, VALUE);
+extern VALUE Info_undefine(VALUE, VALUE, VALUE);
 extern VALUE Info_initialize(VALUE);
 extern VALUE rm_info_new(void);
 
@@ -565,6 +673,7 @@ ATTR_ACCESSOR(Image, blur)
 ATTR_ACCESSOR(Image, border_color)
 ATTR_READER(Image, bounding_box)
 ATTR_ACCESSOR(Image, chromaticity)
+ATTR_ACCESSOR(Image, class_type)
 ATTR_WRITER(Image, clip_mask)
 ATTR_ACCESSOR(Image, color_profile)
 ATTR_READER(Image, colors)
@@ -594,22 +703,26 @@ ATTR_ACCESSOR(Image, matte)
 ATTR_ACCESSOR(Image, matte_color)
 ATTR_READER(Image, mean_error_per_pixel)
 ATTR_READER(Image, mime_type)
+ATTR_WRITER(Image, monitor)
 ATTR_ACCESSOR(Image, montage)
 ATTR_READER(Image, normalized_mean_error)
 ATTR_READER(Image, normalized_maximum_error)
 ATTR_READER(Image, number_colors)
 ATTR_ACCESSOR(Image, offset)
 ATTR_WRITER(Image, opacity)
+ATTR_READER(Image, orientation)
 ATTR_ACCESSOR(Image, page)
+ATTR_READER(Image, quality)
 ATTR_READER(Image, quantum_depth)
 ATTR_ACCESSOR(Image, rendering_intent)
 ATTR_READER(Image, rows)
 ATTR_READER(Image, scene)
 ATTR_ACCESSOR(Image, start_loop)
-ATTR_ACCESSOR(Image, class_type)
+ATTR_ACCESSOR(Image, ticks_per_second)
 ATTR_ACCESSOR(Image, tile_info)
 ATTR_READER(Image, total_colors)
 ATTR_ACCESSOR(Image, units)
+ATTR_ACCESSOR(Image, virtual_pixel_method)
 ATTR_ACCESSOR(Image, x_resolution)
 ATTR_ACCESSOR(Image, y_resolution)
 
@@ -623,6 +736,7 @@ extern VALUE Image_initialize(VALUE, VALUE, VALUE, VALUE, VALUE);
 
 extern VALUE Image_adaptive_threshold(int, VALUE *, VALUE);
 extern VALUE Image_add_noise(VALUE, VALUE);
+extern VALUE Image_add_noise_channel(int, VALUE *, VALUE);
 extern VALUE Image_affine_transform(VALUE, VALUE);
 extern VALUE Image_aref(VALUE, VALUE);
 extern VALUE Image_aset(VALUE, VALUE, VALUE);
@@ -630,13 +744,14 @@ extern VALUE Image_properties(VALUE);
 extern VALUE Image_bilevel_channel(int, VALUE *, VALUE);
 extern VALUE Image_black_threshold(int, VALUE *, VALUE);
 extern VALUE Image_blur_image(int, VALUE *, VALUE);
+extern VALUE Image_blur_channel(int, VALUE *, VALUE);
 extern VALUE Image_border(VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_border_bang(VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_capture(int, VALUE *, VALUE);
 extern VALUE Image_change_geometry(VALUE, VALUE);
 extern VALUE Image_changed_q(VALUE);
 extern VALUE Image_channel(VALUE, VALUE);
-extern VALUE Image_channel_compare(int, VALUE *, VALUE);
+extern VALUE Image_compare_channel(int, VALUE *, VALUE);
 extern VALUE Image_channel_depth(int, VALUE *, VALUE);
 extern VALUE Image_channel_extrema(int, VALUE *, VALUE);
 extern VALUE Image_channel_mean(int, VALUE *, VALUE);
@@ -664,6 +779,7 @@ extern VALUE Image_despeckle(VALUE);
 extern VALUE Image_difference(VALUE, VALUE);
 extern VALUE Image_dispatch(int, VALUE *, VALUE);
 extern VALUE Image_display(VALUE);
+extern VALUE Image_distortion_channel(int, VALUE *, VALUE);
 extern VALUE Image__dump(VALUE, VALUE);
 extern VALUE Image_dup(VALUE);
 extern VALUE Image_each_profile(VALUE);
@@ -672,7 +788,8 @@ extern VALUE Image_emboss(int, VALUE *, VALUE);
 extern VALUE Image_enhance(VALUE);
 extern VALUE Image_equalize(VALUE);
 extern VALUE Image_erase_bang(VALUE);
-extern VALUE Image_export_pixels(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
+extern VALUE Image_export_pixels(int, VALUE *, VALUE);
+extern VALUE Image_export_pixels_to_str(int, VALUE *, VALUE);
 extern VALUE Image_flip(VALUE);
 extern VALUE Image_flip_bang(VALUE);
 extern VALUE Image_flop(VALUE);
@@ -687,7 +804,7 @@ extern VALUE Image_get_pixels(VALUE, VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_gray_q(VALUE);
 extern VALUE Image_grayscale_pseudo_class(int, VALUE *, VALUE);
 extern VALUE Image_implode(int, VALUE *, VALUE);
-extern VALUE Image_import_pixels(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
+extern VALUE Image_import_pixels(int, VALUE *, VALUE);
 extern VALUE Image_init_copy(VALUE, VALUE);
 extern VALUE Image_inspect(VALUE);
 extern VALUE Image_level(int, VALUE *, VALUE);
@@ -706,10 +823,11 @@ extern VALUE Image_motion_blur(VALUE, VALUE, VALUE, VALUE);
 extern VALUE Image_negate(int, VALUE *, VALUE);
 extern VALUE Image_negate_channel(int, VALUE *, VALUE);
 extern VALUE Image_normalize(VALUE);
+extern VALUE Image_normalize_channel(int, VALUE *, VALUE);
 extern VALUE Image_oil_paint(int, VALUE *, VALUE);
 extern VALUE Image_opaque(VALUE, VALUE, VALUE);
 extern VALUE Image_opaque_q(VALUE);
-extern VALUE Image_ordered_dither(VALUE);
+extern VALUE Image_ordered_dither(int, VALUE *, VALUE);
 extern VALUE Image_palette_q(VALUE);
 extern VALUE Image_ping(VALUE, VALUE);
 extern VALUE Image_pixel_color(int, VALUE *, VALUE);
@@ -725,6 +843,7 @@ extern VALUE Image_raise(int, VALUE *, VALUE);
 extern VALUE Image_random_channel_threshold(VALUE, VALUE, VALUE);
 extern VALUE Image_random_threshold_channel(int, VALUE *, VALUE);
 extern VALUE Image_read(VALUE, VALUE);
+extern VALUE Image_read_inline(VALUE, VALUE);
 extern VALUE Image_reduce_noise(VALUE, VALUE);
 extern VALUE Image_resize(int, VALUE *, VALUE);
 extern VALUE Image_resize_bang(int, VALUE *, VALUE);
@@ -736,14 +855,20 @@ extern VALUE Image_sample_bang(int, VALUE *, VALUE);
 extern VALUE Image_scale(int, VALUE *, VALUE);
 extern VALUE Image_scale_bang(int, VALUE *, VALUE);
 extern VALUE Image_segment(int, VALUE *, VALUE);
+extern VALUE Image_sepiatone(int, VALUE *, VALUE);
+extern VALUE Image_set_channel_depth(VALUE, VALUE, VALUE);
 extern VALUE Image_shade(int, VALUE *, VALUE);
+extern VALUE Image_shadow(int, VALUE *, VALUE);
 extern VALUE Image_sharpen(int, VALUE *, VALUE);
+extern VALUE Image_sharpen_channel(int, VALUE *, VALUE);
 extern VALUE Image_shave(VALUE, VALUE, VALUE);
 extern VALUE Image_shave_bang(VALUE, VALUE, VALUE);
 extern VALUE Image_shear(VALUE, VALUE, VALUE);
+extern VALUE Image_sigmoidal_contrast_channel(int, VALUE *, VALUE);
 extern VALUE Image_signature(VALUE);
 extern VALUE Image_solarize(int, VALUE *, VALUE);
 extern VALUE Image_spaceship(VALUE, VALUE);
+extern VALUE Image_splice(int, VALUE *, VALUE);
 extern VALUE Image_spread(int, VALUE *, VALUE);
 extern VALUE Image_statistics(VALUE);
 extern VALUE Image_stegano(VALUE, VALUE, VALUE);
@@ -761,7 +886,9 @@ extern VALUE Image_to_color(VALUE, VALUE);
 extern VALUE Image_transparent(int, VALUE *, VALUE);
 extern VALUE Image_trim(VALUE);
 extern VALUE Image_trim_bang(VALUE);
-extern VALUE Image_unsharp_mask(VALUE, VALUE, VALUE, VALUE, VALUE);
+extern VALUE Image_unsharp_mask(int, VALUE *, VALUE);
+extern VALUE Image_unsharp_mask_channel(int, VALUE *, VALUE);
+extern VALUE Image_vignette(int, VALUE *, VALUE);
 extern VALUE Image_wave(int, VALUE *, VALUE);
 extern VALUE Image_white_threshold(int, VALUE *, VALUE);
 extern VALUE Image_write(VALUE, VALUE);
@@ -797,7 +924,9 @@ extern VALUE  Color_to_s(VALUE);
 extern VALUE  Color_from_ColorInfo(const ColorInfo *);
 extern VALUE  ClassType_new(ClassType);
 extern VALUE  ColorspaceType_new(ColorspaceType);
+extern VALUE  CompositeOperator_new(CompositeOperator);
 extern VALUE  CompressionType_new(CompressionType);
+extern VALUE  DisposeType_new(DisposeType);
 extern VALUE  EndianType_new(EndianType);
 extern VALUE  FilterTypes_new(FilterTypes);
 extern VALUE  Font_to_s(VALUE);
@@ -815,6 +944,10 @@ ATTR_ACCESSOR(Pixel, red)
 ATTR_ACCESSOR(Pixel, green)
 ATTR_ACCESSOR(Pixel, blue)
 ATTR_ACCESSOR(Pixel, opacity)
+ATTR_ACCESSOR(Pixel, cyan)
+ATTR_ACCESSOR(Pixel, magenta)
+ATTR_ACCESSOR(Pixel, yellow)
+ATTR_ACCESSOR(Pixel, black)
 extern VALUE  Pixel_case_eq(VALUE, VALUE);
 extern VALUE  Pixel_clone(VALUE);
 extern VALUE  Pixel_dup(VALUE);
@@ -856,6 +989,11 @@ extern VALUE  TypeMetric_from_TypeMetric(TypeMetric *);
 #if defined(HAVE_GETIMAGESTATISTICS)
 extern VALUE  Statistics_new(ImageStatistics *);
 #endif
+#if defined(HAVE_IMAGE_ORIENTATION)
+extern VALUE   OrientationType_new(OrientationType);
+#endif
+extern const char *StorageType_name(StorageType);
+extern VALUE   VirtualPixelMethod_new(VirtualPixelMethod);
 
 #if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
 extern VALUE  Enum_alloc(VALUE);
@@ -881,10 +1019,11 @@ extern VALUE  rm_no_freeze(VALUE);
 extern char  *rm_string_value_ptr(volatile VALUE *);
 #endif
 extern char  *rm_string_value_ptr_len(volatile VALUE *, long *);
+extern int    rm_strcasecmp(const char *, const char *);
 extern void   rm_check_ary_len(VALUE, int);
 extern void   rm_check_frozen(VALUE);
-extern VALUE  rm_obj_to_s(VALUE);
-extern double rm_fuzz_to_dbl(VALUE fuzz);
+extern double rm_fuzz_to_dbl(VALUE);
+extern double rm_percentage(VALUE);
 extern VALUE  rm_define_enum_type(char *);
 extern void   rm_write_temp_image(Image *, char *);
 extern void   rm_delete_temp_image(char *);
@@ -893,8 +1032,11 @@ extern void   rm_handle_error(ExceptionInfo *);
 extern void   rm_handle_all_errors(Image *);
 extern void   rm_attr_write(VALUE, VALUE);
 extern void   rm_get_geometry(VALUE, long *, long *, unsigned long *, unsigned long *, int *);
-extern Image *rm_toseq(VALUE);
-extern void   rm_unseq(Image *);
+extern void   rm_split(Image *);
+
+#if defined(HAVE_SETIMAGEPROGRESSMONITOR)
+extern MagickBooleanType rm_progress_monitor(const char *, const MagickOffsetType, const MagickSizeType, void *);
+#endif
 
 #endif
 
