@@ -1,6 +1,6 @@
-/* $Id: rmdraw.c,v 1.15 2004/07/31 22:04:09 rmagick Exp $ */
+/* $Id: rmdraw.c,v 1.29 2006/02/19 17:12:20 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2004 by Timothy P. Hunter
+|                Copyright (C) 2006 by Timothy P. Hunter
 | Name:     rmdraw.c
 | Author:   Tim Hunter
 | Purpose:  Contains Draw class methods.
@@ -8,11 +8,16 @@
 \============================================================================*/
 
 #include "rmagick.h"
+#include "float.h"
 
 static void mark_Draw(void *);
 static void destroy_Draw(void *);
 static void destroy_Montage(void *);
 
+typedef MagickBooleanType (get_type_metrics_func_t)(Image *, const DrawInfo *, TypeMetric *);
+static VALUE get_type_metrics(int, VALUE *, VALUE, get_type_metrics_func_t *);
+
+
 /*
     Method:     Draw#affine=
     Purpose:    set the affine matrix from an Magick::AffineMatrix
@@ -43,6 +48,21 @@ Draw_align_eq(VALUE self, VALUE align)
     return self;
 }
 
+/*
+    Method:     Draw#border_color=
+    Purpose:    border_color attribute writer
+*/
+VALUE
+Draw_border_color_eq(VALUE self, VALUE border_color)
+{
+    Draw *draw;
+
+    rm_check_frozen(self);
+    Data_Get_Struct(self, Draw, draw);
+    Color_to_PixelPacket(&draw->info->border_color, border_color);
+    return self;
+}
+
 /*
     Method:     Draw#decorate=
     Purpose:    decorate attribute writer
@@ -283,7 +303,7 @@ Draw_rotation_eq(VALUE self, VALUE deg)
     Data_Get_Struct(self, Draw, draw);
 
     degrees = NUM2DBL(deg);
-    if (degrees != 0.0)
+    if (fabs(degrees) > DBL_EPSILON)
     {
         affine.sx=1.0;
         affine.rx=0.0;
@@ -401,14 +421,22 @@ VALUE Draw_annotate(
     // allowing the app a chance to modify the object's attributes
     if (rb_block_given_p())
     {
-        rb_obj_instance_eval(0, NULL, self);
+        (void)rb_obj_instance_eval(0, NULL, self);
     }
 
     rm_check_frozen(ImageList_cur_image(image_arg));
     Data_Get_Struct(ImageList_cur_image(image_arg), Image, image);
 
-    // Copy text to Draw structure
-    magick_clone_string(&draw->info->text, STRING_PTR(text));
+    // Translate & store in Draw structure
+#if defined(HAVE_INTERPRETIMAGEATTRIBUTES)
+    draw->info->text = InterpretImageAttributes(NULL, image, STRING_PTR(text));
+#else
+    draw->info->text = TranslateText(NULL, image, STRING_PTR(text));
+#endif
+    if (!draw->info->text)
+    {
+           rb_raise(rb_eArgError, "no text");
+    }
 
     // Create geometry string, copy to Draw structure, overriding
     // any previously existing value.
@@ -439,7 +467,7 @@ VALUE Draw_annotate(
 
     draw->info->affine = keep;
 
-    HANDLE_IMG_ERROR(image)
+    HANDLE_ERROR_IMG(image)
 
     return self;
 }
@@ -457,7 +485,7 @@ Draw_clone(VALUE self)
     clone = Draw_dup(self);
     if (OBJ_FROZEN(self))
     {
-        rb_obj_freeze(clone);
+        (void)rb_obj_freeze(clone);
     }
 
     return clone;
@@ -479,6 +507,7 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
     CompositeOperator cop = OverCompositeOp;
     volatile VALUE image;
     Image *comp_img;
+    struct TmpFile_Name *tmpfile;
     char name[MaxTextExtent];
                             // Buffer for "image" primitive
     char primitive[MaxTextExtent];
@@ -569,12 +598,13 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
     Data_Get_Struct(image, Image, comp_img);
     rm_write_temp_image(comp_img, name);
 
-    // Add the temp filename to the filename array
-    if (!draw->tmpfile_ary)
-    {
-        draw->tmpfile_ary = rb_ary_new();
-    }
-    rb_ary_push(draw->tmpfile_ary, rb_str_new2(name));
+    // Add the temp filename to the filename array.
+    // Use Magick storage since we need to keep the list around
+    // until destroy_Draw is called.
+    tmpfile = magick_malloc(sizeof(struct TmpFile_Name)+strlen(name));
+    strcpy(tmpfile->name, name);
+    tmpfile->next = draw->tmpfile_ary;
+    draw->tmpfile_ary = tmpfile;
 
     // Form the drawing primitive
     (void) sprintf(primitive, "image %s %g,%g,%g,%g '%s'", op, x, y, width, height, name);
@@ -610,7 +640,7 @@ Draw_draw(VALUE self, VALUE image_arg)
     magick_clone_string(&(draw->info->primitive), STRING_PTR(draw->primitives));
 
     (void) DrawImage(image, draw->info);
-    HANDLE_IMG_ERROR(image)
+    HANDLE_ERROR_IMG(image)
 
     magick_free(draw->info->primitive);
     draw->info->primitive = NULL;
@@ -631,11 +661,11 @@ Draw_dup(VALUE self)
     volatile VALUE dup;
 
     draw = ALLOC(Draw);
-    memset(draw, '\0', sizeof(Draw));
+    memset(draw, 0, sizeof(Draw));
     dup = Data_Wrap_Struct(CLASS_OF(self), mark_Draw, destroy_Draw, draw);
     if (rb_obj_tainted(self))
     {
-        rb_obj_taint(dup);
+        (void)rb_obj_taint(dup);
     }
     return rb_funcall(dup, ID_initialize_copy, 1, self);
 }
@@ -643,106 +673,34 @@ Draw_dup(VALUE self)
 
 /*
     Method:     Draw#get_type_metrics([image, ]text)
+                Draw#get_multiline_type_metrics([image, ]text)
     Purpose:    returns measurements for a given font and text string
     Notes:      If the image argument has been omitted, use a dummy
                 image, but make sure the text has none of the special
                 characters that refer to image attributes.
 */
-
-static VALUE get_dummy_tm_img(VALUE klass)
-{
-    volatile VALUE dummy_img = 0;
-    Info *info;
-    Image *image;
-
-    if (rb_cvar_defined(klass, ID__dummy_img_) != Qtrue)
-    {
-
-        info = CloneImageInfo(NULL);
-        if (!info)
-        {
-            rb_raise(rb_eNoMemError, "not enough memory to continue");
-        }
-        image = AllocateImage(info);
-        if (!image)
-        {
-            rb_raise(rb_eNoMemError, "not enough memory to continue");
-        }
-        DestroyImageInfo(info);
-        dummy_img = rm_image_new(image);
-
-        RUBY18(rb_cvar_set(klass, ID__dummy_img_, dummy_img, 0));
-        RUBY16(rb_cvar_set(klass, ID__dummy_img_, dummy_img));
-    }
-    dummy_img = rb_cvar_get(klass, ID__dummy_img_);
-
-    return dummy_img;
-}
-
-
 VALUE
 Draw_get_type_metrics(
     int argc,
     VALUE *argv,
     VALUE self)
 {
-    static char attrs[] = "bcdefghiklmnopqrstuwxyz";
-#define ATTRS_L (sizeof(attrs)-1)
-    Image *image;
-    Draw *draw;
-    TypeMetric metrics;
-    char *text;
-    long text_l;
-    int x;
-    unsigned int okay;
+    return get_type_metrics(argc, argv, self, GetTypeMetrics);
+}
 
-    switch (argc)
-    {
-        case 1:                   // use default image
-            text = STRING_PTR_LEN(argv[0], text_l);
 
-            for (x = 0; x < text_l; x++)
-            {
-                // Ensure text string doesn't refer to image attributes.
-                if (text[x] == '%' && x < text_l-1)
-                {
-                    int y;
-                    char spec = text[x+1];
-
-                    for (y = 0; y < ATTRS_L; y++)
-                    {
-                        if (spec == attrs[y])
-                        {
-                            rb_raise(rb_eArgError,
-                                "text string contains image attribute reference `%%%c'",
-                                spec);
-                        }
-                    }
-                }
-            }
-
-            Data_Get_Struct(get_dummy_tm_img(CLASS_OF(self)), Image, image);
-            break;
-        case 2:
-            Data_Get_Struct(ImageList_cur_image(argv[0]), Image, image);
-            text = STRING_PTR(argv[1]);
-            break;                  // okay
-        default:
-            rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
-            break;
-    }
-
-    Data_Get_Struct(self, Draw, draw);
-    magick_clone_string(&draw->info->text, text);
-
-    okay = GetTypeMetrics(image, draw->info, &metrics);
-
-    if (!okay)
-    {
-        rb_warning("RMagick: get_type_metrics failed");
-        return Qnil;
-    }
-    return TypeMetric_from_TypeMetric(&metrics);
+VALUE
+Draw_get_multiline_type_metrics(
+     int argc,
+     VALUE *argv,
+     VALUE self)
+{
+#if defined(HAVE_GETMULTILINETYPEMETRICS)
+     return get_type_metrics(argc, argv, self, GetMultilineTypeMetrics);
+#else
+     rm_not_implemented();
+     return (VALUE)0;
+#endif
 }
 
 /*
@@ -804,8 +762,8 @@ Draw_initialize(VALUE self)
     Data_Get_Struct(info_obj, Info, info);
     draw->info = CloneDrawInfo(info, NULL);
 
-    draw->primitives = 0;
-    draw->tmpfile_ary = 0;
+    draw->primitives = (VALUE)0;
+    draw->tmpfile_ary = NULL;
 
     return self;
 }
@@ -839,7 +797,7 @@ Draw_new(VALUE class)
     volatile VALUE draw_obj;
 
     draw = ALLOC(Draw);
-    memset(draw, '\0', sizeof(Draw));
+    memset(draw, 0, sizeof(Draw));
     draw_obj = Data_Wrap_Struct(class, mark_Draw, destroy_Draw, draw);
 
 #if !defined(HAVE_RB_DEFINE_ALLOC_FUNC)
@@ -862,7 +820,7 @@ Draw_primitive(VALUE self, VALUE primitive)
     rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
 
-    if (!draw->primitives)
+    if (draw->primitives == (VALUE)0)
     {
         draw->primitives = primitive;
     }
@@ -884,11 +842,7 @@ mark_Draw(void *drawptr)
 {
     Draw *draw = (Draw *)drawptr;
 
-    if (draw->tmpfile_ary)
-    {
-        rb_gc_mark(draw->tmpfile_ary);
-    }
-    if (draw->primitives)
+    if (draw->primitives != (VALUE)0)
     {
         rb_gc_mark(draw->primitives);
     }
@@ -902,17 +856,17 @@ static void
 destroy_Draw(void *drawptr)
 {
     Draw *draw = (Draw *)drawptr;
-    volatile VALUE tmpfile;
+    struct TmpFile_Name *tmpfile;
 
     DestroyDrawInfo(draw->info);
 
     // Erase any temporary image files.
-    if (draw->tmpfile_ary)
+    while (draw->tmpfile_ary)
     {
-        while ((tmpfile = rb_ary_shift(draw->tmpfile_ary)) != Qnil)
-        {
-            rm_delete_temp_image(STRING_PTR(tmpfile));
-        }
+        tmpfile = draw->tmpfile_ary;
+        draw->tmpfile_ary = draw->tmpfile_ary->next;
+        rm_delete_temp_image(tmpfile->name);
+        magick_free(tmpfile);
     }
 
     xfree(drawptr);
@@ -1273,3 +1227,118 @@ Montage_title_eq(VALUE self, VALUE title)
     return self;
 }
 
+
+static VALUE get_dummy_tm_img(VALUE klass)
+{
+    volatile VALUE dummy_img = 0;
+    Info *info;
+    Image *image;
+
+    if (rb_cvar_defined(klass, ID__dummy_img_) != Qtrue)
+    {
+
+        info = CloneImageInfo(NULL);
+        if (!info)
+        {
+            rb_raise(rb_eNoMemError, "not enough memory to continue");
+        }
+        image = AllocateImage(info);
+        if (!image)
+        {
+            rb_raise(rb_eNoMemError, "not enough memory to continue");
+        }
+        DestroyImageInfo(info);
+        dummy_img = rm_image_new(image);
+
+        RUBY18(rb_cvar_set(klass, ID__dummy_img_, dummy_img, 0));
+        RUBY16(rb_cvar_set(klass, ID__dummy_img_, dummy_img));
+    }
+    dummy_img = rb_cvar_get(klass, ID__dummy_img_);
+
+    return dummy_img;
+}
+
+
+/*
+ *  Static:     get_type_metrics
+ *  Purpose:    Call a get-type-metrics function
+ *  Notes:      called by Draw_get_type_metrics and Draw_get_multiline_type_metrics
+*/
+static VALUE
+get_type_metrics(
+     int argc,
+     VALUE *argv,
+     VALUE self,
+     get_type_metrics_func_t *getter)
+{
+     static char attrs[] = "OPbcdefghiklmnopqrstuwxyz[@#%";
+ #define ATTRS_L (sizeof(attrs)-1)
+     Image *image;
+     Draw *draw;
+     TypeMetric metrics;
+     char *text = NULL;
+     long text_l;
+     long x;
+     unsigned int okay;
+
+     switch (argc)
+     {
+         case 1:                   // use default image
+             text = STRING_PTR_LEN(argv[0], text_l);
+
+             for (x = 0; x < text_l; x++)
+             {
+                 // Ensure text string doesn't refer to image attributes.
+                 if (text[x] == '%' && x < text_l-1)
+                 {
+                     int y;
+                     char spec = text[x+1];
+
+                     for (y = 0; y < ATTRS_L; y++)
+                     {
+                         if (spec == attrs[y])
+                         {
+                             rb_raise(rb_eArgError,
+                                 "text string contains image attribute reference `%%%c'",
+                                 spec);
+                         }
+                     }
+                 }
+             }
+
+             Data_Get_Struct(get_dummy_tm_img(CLASS_OF(self)), Image, image);
+             break;
+         case 2:
+             Data_Get_Struct(ImageList_cur_image(argv[0]), Image, image);
+             text = STRING_PTR_LEN(argv[1], text_l);
+             break;                  // okay
+         default:
+             rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
+             break;
+     }
+
+     if (text_l == 0)
+     {
+         rb_raise(rb_eArgError, "no text to measure");
+     }
+
+     Data_Get_Struct(self, Draw, draw);
+#if defined(HAVE_INTERPRETIMAGEATTRIBUTES)
+     draw->info->text = InterpretImageAttributes(NULL, image, text);
+#else
+     draw->info->text = TranslateText(NULL, image, text);
+#endif
+     if (!draw->info->text)
+     {
+         rb_raise(rb_eArgError, "no text to measure");
+     }
+
+     okay = (*getter)(image, draw->info, &metrics);
+
+     if (!okay)
+     {
+         rb_raise(rb_eRuntimeError, "Can't measure text. Are the fonts installed? "
+                  "Is the FreeType library installed?");
+     }
+     return TypeMetric_from_TypeMetric(&metrics);
+}
