@@ -1,6 +1,6 @@
-/* $Id: rmilist.c,v 1.15 2004/06/19 20:41:18 rmagick Exp $ */
+/* $Id: rmilist.c,v 1.31 2006/02/03 23:26:05 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2004 by Timothy P. Hunter
+|                Copyright (C) 2006 by Timothy P. Hunter
 | Name:     rmilist.c
 | Author:   Tim Hunter
 | Purpose:  ImageList class method definitions for RMagick
@@ -8,6 +8,7 @@
 
 #include "rmagick.h"
 
+
 /*
     Method:     ImageList#animate(<delay>)
     Purpose:    repeatedly display the images in the images array to an XWindow
@@ -23,7 +24,7 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
     volatile VALUE info_obj;
 
     // Convert the images array to an images sequence.
-    images = rm_toseq(self);
+    images = rm_images_from_imagelist(self);
 
     if (argc > 1)
     {
@@ -35,7 +36,7 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
         unsigned int delay;
 
         delay = NUM2UINT(argv[0]);
-        for (img = images; img; img = GET_NEXT_IMAGE(img))
+        for (img = images; img; img = GetNextImageInList(img))
         {
             img->delay = delay;
         }
@@ -48,7 +49,7 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
 
     (void) AnimateImages(info, images);
     rm_handle_all_errors(images);
-    rm_unseq(images);
+    rm_split(images);
 
     return self;
 }
@@ -66,7 +67,7 @@ ImageList_append(VALUE self, VALUE stack_arg)
     ExceptionInfo exception;
 
     // Convert the image array to an image sequence.
-    images = rm_toseq(self);
+    images = rm_images_from_imagelist(self);
 
     // If stack == true, stack rectangular images top-to-bottom,
     // otherwise left-to-right.
@@ -74,8 +75,8 @@ ImageList_append(VALUE self, VALUE stack_arg)
 
     GetExceptionInfo(&exception);
     result = AppendImages(images, stack, &exception);
+    rm_split(images);
     HANDLE_ERROR
-    rm_unseq(images);
 
     return rm_image_new(result);
 }
@@ -92,12 +93,12 @@ ImageList_average(VALUE self)
     ExceptionInfo exception;
 
     // Convert the images array to an images sequence.
-    images = rm_toseq(self);
+    images = rm_images_from_imagelist(self);
 
     GetExceptionInfo(&exception);
     result = AverageImages(images, &exception);
     rm_handle_all_errors(images);
-    rm_unseq(images);
+    rm_split(images);
 
     return rm_image_new(result);
 }
@@ -113,64 +114,41 @@ ImageList_average(VALUE self)
 VALUE
 ImageList_coalesce(VALUE self)
 {
-    Image *images, *results, *result, *next;
-    volatile VALUE new_imagelist;
+    Image *images, *results;
     ExceptionInfo exception;
 
     // Convert the image array to an image sequence.
-    images = rm_toseq(self);
+    images = rm_images_from_imagelist(self);
 
     GetExceptionInfo(&exception);
     results = CoalesceImages(images, &exception);
+    rm_split(images);
     HANDLE_ERROR
-    rm_unseq(images);
 
-    new_imagelist = rm_imagelist_new();
-
-    // CoalesceImages returns an image sequence. Create a
-    // new ImageList and store the images in the array.
-#if HAVE_REMOVEFIRSTIMAGEFROMLIST
-    result = results;
-    while (result)
-    {
-        next = RemoveFirstImageFromList(&result);
-        rm_imagelist_push(new_imagelist, rm_image_new(next));
-    }
-#else
-    for (result = results; result; result = next)
-    {
-        next = GET_NEXT_IMAGE(result);
-        result->previous = result->next = NULL;
-        rm_imagelist_push(new_imagelist, rm_image_new(result));
-    }
-#endif
-
-    // Set new_imagelist.scene = 0
-    rb_iv_set(new_imagelist, "@scene", INT2FIX(0));
-
-    return new_imagelist;
+    return rm_imagelist_from_images(results);
 }
 
+
 /*
     Method:     ImageList#deconstruct
     Purpose:    compares each image with the next in a sequence and returns
                 the maximum bounding region of any pixel differences it
                 discovers.
-    Returns:    a new image, or nil
+    Returns:    a new imagelist
 */
 VALUE
 ImageList_deconstruct(VALUE self)
 {
-    Image *images, *new_image;
+    Image *new_images, *images;
     ExceptionInfo exception;
 
-    images = rm_toseq(self);
+    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
-    new_image = DeconstructImages(images, &exception);
+    new_images = DeconstructImages(images, &exception);
+    rm_split(images);
     HANDLE_ERROR
-    rm_unseq(images);
 
-    return rm_image_new(new_image);
+    return rm_imagelist_from_images(new_images);
 }
 
 /*
@@ -186,7 +164,7 @@ ImageList_display(VALUE self)
     unsigned int ok;
 
     // Convert the images array to an images sequence.
-    images = rm_toseq(self);
+    images = rm_images_from_imagelist(self);
 
     // Create a new Info object to use with this call
     info_obj = rm_info_new();
@@ -197,7 +175,7 @@ ImageList_display(VALUE self)
     {
         rm_handle_all_errors(images);
     }
-    rm_unseq(images);
+    rm_split(images);
 
     return self;
 }
@@ -214,11 +192,11 @@ ImageList_flatten_images(VALUE self)
     Image *images, *new_image;
     ExceptionInfo exception;
 
-    images = rm_toseq(self);
+    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
     new_image = FlattenImages(images, &exception);
+    rm_split(images);
     HANDLE_ERROR
-    rm_unseq(images);
 
     return rm_image_new(new_image);
 }
@@ -227,16 +205,15 @@ ImageList_flatten_images(VALUE self)
 /*
     Method:     ImageList#map
     Purpose:    Call MapImages
-    Returns:    a new Image with mapped images. @scene is set to self.scene
+    Returns:    a new ImageList with mapped images. @scene is set to self.scene
 */
 VALUE
 ImageList_map(VALUE self, VALUE map_image, VALUE dither_arg)
 {
-    Image *images, *new_images = NULL;
-    Image *new_image;
+    Image *images, *clone_images = NULL;
     Image *map;
     unsigned int dither;
-    volatile VALUE image, image_obj, scene;
+    volatile VALUE image, scene, new_imagelist;
     ExceptionInfo exception;
 
     image = ImageList_cur_image(map_image);
@@ -248,31 +225,23 @@ ImageList_map(VALUE self, VALUE map_image, VALUE dither_arg)
     }
 
     // Convert image array to image sequence, clone image sequence.
-    images = rm_toseq(self);
+    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
-    new_images = CloneImageList(images, &exception);
+    clone_images = CloneImageList(images, &exception);
+    rm_split(images);
     HANDLE_ERROR
-    rm_unseq(images);
 
     // Call ImageMagick
     dither = !(dither_arg == Qfalse || dither_arg == Qnil);
-    (void) MapImages(new_images, map, dither);
-    HANDLE_IMG_ERROR(new_images)
-
-    // Create new ImageList object, convert mapped image sequence to images, append
-    // to imagelist.
-    image_obj = rm_imagelist_new();
-    while ((new_image = ShiftImageList(&new_images)))
-    {
-        rm_imagelist_push(image_obj, rm_image_new(new_image));
-    }
-
+    (void) MapImages(clone_images, map, dither);
+    HANDLE_ERROR_IMG(clone_images)
 
     // Set @scene in new ImageList object to same value as in self.
+    new_imagelist = rm_imagelist_from_images(clone_images);
     scene = rb_iv_get(self, "@scene");
-    rb_iv_set(image_obj, "@scene", scene);
+    (void) rm_imagelist_scene_eq(new_imagelist, scene);
 
-    return image_obj;
+    return new_imagelist;
 }
 
 /*
@@ -286,8 +255,7 @@ ImageList_montage(VALUE self)
 {
     volatile VALUE montage_obj;
     Montage *montage;
-    Image *montage_seq, *next, *image, *image_list;
-    volatile VALUE new_imagelist;
+    Image *montage_seq, *image_list;
     ExceptionInfo exception;
 
     // Create a new instance of the Magick::Montage class
@@ -301,13 +269,13 @@ ImageList_montage(VALUE self)
 
     Data_Get_Struct(montage_obj, Montage, montage);
 
-    image_list = rm_toseq(self);
+    image_list = rm_images_from_imagelist(self);
 
     // If app specified a non-default composition operator, use it for all images.
     if (montage->compose != UndefinedCompositeOp)
     {
         Image *i;
-        for (i = image_list; i; i = GET_NEXT_IMAGE(i))
+        for (i = image_list; i; i = GetNextImageInList(i))
         {
             i->compose = montage->compose;
         }
@@ -317,22 +285,10 @@ ImageList_montage(VALUE self)
 
     // MontageImage can return more than one image.
     montage_seq = MontageImages(image_list, montage->info, &exception);
+    rm_split(image_list);
     HANDLE_ERROR
-    rm_unseq(image_list);
 
-    // Construct a new image and store the image(s) in its images array.
-    new_imagelist = rm_imagelist_new();
-    for (image = montage_seq; image; image = next)
-    {
-        next = GET_NEXT_IMAGE(image);
-        image->previous = image->next = NULL;
-        rm_imagelist_push(new_imagelist, rm_image_new(image));
-    }
-
-    // img.scene = 0
-    rb_iv_set(new_imagelist, "@scene", INT2FIX(0));
-
-    return new_imagelist;
+    return rm_imagelist_from_images(montage_seq);
 }
 
 /*
@@ -346,38 +302,29 @@ ImageList_montage(VALUE self)
 VALUE
 ImageList_morph(VALUE self, VALUE nimages)
 {
-    Image *images, *new_images, *next, *new;
+    Image *images, *new_images;
     ExceptionInfo exception;
-    volatile VALUE new_imagelist;
-    unsigned long number_images;
+    long number_images;
 
     if (rm_imagelist_length(self) < 1)
     {
         rb_raise(rb_eArgError, "no images in this image list");
     }
-    number_images = NUM2ULONG(nimages);
+
+    // Use a signed long so we can test for a negative argument.
+    number_images = NUM2LONG(nimages);
     if (number_images <= 0)
     {
         rb_raise(rb_eArgError, "number of intervening images must be > 0");
     }
 
-    images = rm_toseq(self);
+    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
-    new_images = MorphImages(images, number_images, &exception);
+    new_images = MorphImages(images, (unsigned long)number_images, &exception);
+    rm_split(images);
     HANDLE_ERROR
 
-    new_imagelist = rm_imagelist_new();
-    for (new = new_images; new; new = next)
-    {
-        next = GET_NEXT_IMAGE(new);
-        new->previous = new->next = NULL;
-        rm_imagelist_push(new_imagelist, rm_image_new(new));
-    }
-
-    // new_imagelist.scene = 0
-    rb_iv_set(new_imagelist, "@scene", INT2FIX(0));
-
-    return new_imagelist;
+    return rm_imagelist_from_images(new_images);
 }
 
 /*
@@ -391,18 +338,19 @@ ImageList_mosaic(VALUE self)
     Image *images, *new_image;
     ExceptionInfo exception;
 
-    images = rm_toseq(self);
+    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
     new_image = MosaicImages(images, &exception);
+    rm_split(images);
     HANDLE_ERROR
-    rm_unseq(images);
 
     return rm_image_new(new_image);
 }
 
 /*
     External:   rm_imagelist_new
-    Purpose:    create a new ImageList object
+    Purpose:    create a new ImageList object with no images
+    Notes:      this simply calls ImageList.new() in RMagick.rb
 */
 VALUE
 rm_imagelist_new()
@@ -410,6 +358,75 @@ rm_imagelist_new()
     return rb_funcall(Class_ImageList, ID_new, 0);
 }
 
+
+
+/*
+     Extern:   rm_imagelist_from_images
+     Purpose:  Construct a new imagelist object from a list of images
+     Notes:    Sets @scene to 0.
+*/
+VALUE
+rm_imagelist_from_images(Image *images)
+{
+     volatile VALUE new_imagelist;
+     Image *image;
+
+     new_imagelist = rm_imagelist_new();
+
+     while (images)
+     {
+          image = RemoveFirstImageFromList(&images);
+          rm_imagelist_push(new_imagelist, rm_image_new(image));
+     }
+
+     rb_iv_set(new_imagelist, "@scene", INT2FIX(0));
+     return new_imagelist;
+}
+
+
+/*
+    Extern:     rm_images_from_imagelist
+    Purpose:    Convert an array of Image *s to an ImageMagick scene
+                sequence (i.e. a doubly-linked list of Images)
+    Returns:    a pointer to the head of the scene sequence list
+*/
+Image *
+rm_images_from_imagelist(VALUE imagelist)
+{
+    long x, len;
+    Image *head = NULL;
+
+    Check_Type(imagelist, T_ARRAY);
+    len = rm_imagelist_length(imagelist);
+    if (len == 0)
+    {
+        rb_raise(rb_eArgError, "no images in this image list");
+    }
+
+    for (x = 0; x < len; x++)
+    {
+        Image *image;
+
+        Data_Get_Struct(rb_ary_entry(imagelist, x), Image, image);
+        AppendImageToList(&head, image);
+    }
+
+    return head;
+}
+
+
+/*
+ *   Extern:   rm_imagelist_scene_eq(imagelist, scene)
+ *   Purpose:  @scene attribute writer
+*/
+VALUE
+rm_imagelist_scene_eq(VALUE imagelist, VALUE scene)
+{
+     rm_check_frozen(imagelist);
+     rb_iv_set(imagelist, "@scene", scene);
+     return scene;
+}
+
 /*
     External:   rm_imagelist_length
     Purpose:    return the # of images in an imagelist
@@ -427,11 +444,11 @@ rm_imagelist_length(VALUE imagelist)
     External:   rm_imagelist_push
     Purpose:    push an image onto the end of the imagelist
 */
-VALUE
+void
 rm_imagelist_push(VALUE imagelist, VALUE image)
 {
     rm_check_frozen(imagelist);
-    return rb_funcall(imagelist, ID_push, 1, image);
+    (void) rb_funcall(imagelist, ID_push, 1, image);
 }
 
 /*
@@ -478,10 +495,10 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
 
     // Convert image array to image sequence, clone image sequence.
     GetExceptionInfo(&exception);
-    images = rm_toseq(self);
+    images = rm_images_from_imagelist(self);
     new_images = CloneImageList(images, &exception);
+    rm_split(images);
     HANDLE_ERROR
-    rm_unseq(images);
 
     QuantizeImages(&quantize_info, new_images);
 
@@ -512,6 +529,7 @@ ImageList_to_blob(VALUE self)
     Image *images;
     Info *info;
     volatile VALUE info_obj;
+	volatile VALUE blob_str;
     void *blob = NULL;
     size_t length = 0;
     ExceptionInfo exception;
@@ -520,7 +538,7 @@ ImageList_to_blob(VALUE self)
     Data_Get_Struct(info_obj, Info, info);
 
     // Convert the images array to an images sequence.
-    images = rm_toseq(self);
+    images = rm_images_from_imagelist(self);
 
     GetExceptionInfo(&exception);
     (void) SetImageInfo(info, True, &exception);
@@ -528,7 +546,7 @@ ImageList_to_blob(VALUE self)
     if (*info->magick != '\0')
     {
         Image *img;
-        for (img = images; img; img = GET_NEXT_IMAGE(img))
+        for (img = images; img; img = GetNextImageInList(img))
         {
             strncpy(img->magick, info->magick, sizeof(info->magick)-1);
         }
@@ -539,11 +557,23 @@ ImageList_to_blob(VALUE self)
     // doesn't support multi-image files.
     info->adjoin = True;
     GetExceptionInfo(&exception);
+#if defined(HAVE_IMAGESTOBLOB)
+    blob = ImagesToBlob(info, images, &length, &exception);
+#else
     blob = ImageToBlob(info, images, &length, &exception);
+#endif
+    rm_split(images);
     HANDLE_ERROR
-    rm_unseq(images);
 
-    return (blob && length) ? rb_str_new(blob, length) : Qnil;
+    if (length == 0 || !blob)
+    {
+        return Qnil;
+    }
+
+    blob_str = rb_str_new(blob, length);
+ 	magick_free((void*)blob);
+
+    return blob_str;
 }
 
 
@@ -583,7 +613,7 @@ ImageList_write(VALUE self, VALUE file)
 
 
     // Convert the images array to an images sequence.
-    images = rm_toseq(self);
+    images = rm_images_from_imagelist(self);
 
     if (TYPE(file) == T_FILE)
     {
@@ -596,7 +626,7 @@ ImageList_write(VALUE self, VALUE file)
     else
     {
         // Convert arg to string. Catch exceptions.
-        file = rb_rescue(rm_obj_to_s, file, file_arg_rescue, file);
+        file = rb_rescue(rb_String, file, file_arg_rescue, file);
 
         // Copy the filename to the Info and to the Image.
         filename = STRING_PTR_LEN(file, filenameL);
@@ -608,7 +638,7 @@ ImageList_write(VALUE self, VALUE file)
 
     // Copy the filename into each images. Set a scene number to be used if
     // writing multiple files. (Ref: ImageMagick's utilities/convert.c
-    for (scene = 0, img = images; img; img = GET_NEXT_IMAGE(img))
+    for (scene = 0, img = images; img; img = GetNextImageInList(img))
     {
         img->scene = scene++;
         strcpy(img->filename, info->filename);
@@ -627,9 +657,10 @@ ImageList_write(VALUE self, VALUE file)
         info->adjoin = True;
     }
 
-    for (img = images; img; img = GET_NEXT_IMAGE(img))
+    for (img = images; img; img = GetNextImageInList(img))
     {
         (void) WriteImage(info, img);
+        // images will be split before raising an exception
         rm_handle_all_errors(images);
         if (info->adjoin)
         {
@@ -637,6 +668,6 @@ ImageList_write(VALUE self, VALUE file)
         }
     }
 
-    rm_unseq(images);
+    rm_split(images);
     return self;
 }
