@@ -1,6 +1,6 @@
-/* $Id: rmmain.c,v 1.65 2004/08/18 23:22:36 rmagick Exp $ */
+/* $Id: rmmain.c,v 1.111 2006/02/19 17:12:20 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2004 by Timothy P. Hunter
+|                Copyright (C) 2006 by Timothy P. Hunter
 | Name:     rmmain.c
 | Author:   Tim Hunter
 | Purpose:  Contains all module, class, method declarations.
@@ -10,12 +10,14 @@
 
 #define MAIN                        // Define external variables
 #include "rmagick.h"
+#include "magick/version.h"
 
 /*----------------------------------------------------------------------------\
 | External declarations
 \----------------------------------------------------------------------------*/
 void Init_RMagick(void);
 
+static void test_Magick_version(void);
 static void version_constants(void);
 
 
@@ -74,8 +76,20 @@ Magick_colors(VALUE class)
     unsigned long number_colors, x;
     volatile VALUE ary;
 
+#if defined(HAVE_OLD_GETCOLORINFOLIST)
     color_info_list = GetColorInfoList("*", &number_colors);
 
+#else
+    // IM 6.1.3 added an exception parm to GetColorInfoList
+    ExceptionInfo exception;
+
+    GetExceptionInfo(&exception);
+
+    color_info_list = GetColorInfoList("*", &number_colors, &exception);
+    HANDLE_ERROR
+#endif
+
+
     if (rb_block_given_p())
     {
         for(x = 0; x < number_colors; x++)
@@ -151,8 +165,18 @@ Magick_fonts(VALUE class)
     unsigned long number_types, x;
     volatile VALUE ary;
 
+#if defined(HAVE_OLD_GETTYPEINFOLIST)
     type_info = GetTypeInfoList("*", &number_types);
 
+#else
+    // IM 6.1.3 added an exception argument to GetTypeInfoList
+    ExceptionInfo exception;
+
+    GetExceptionInfo(&exception);
+    type_info = GetTypeInfoList("*", &number_types, &exception);
+    HANDLE_ERROR
+#endif
+
     if (rb_block_given_p())
     {
         for(x = 0; x < number_types; x++)
@@ -269,12 +293,23 @@ Magick_init_formats(VALUE class)
     const MagickInfo **magick_info;
     unsigned long number_formats, x;
     volatile VALUE formats;
+#if !defined(HAVE_OLD_GETMAGICKINFOLIST)
+    ExceptionInfo exception;
+#endif
 
     class = class;      // defeat "never referenced" message from icc
     formats = rb_hash_new();
 
+#if defined(HAVE_OLD_GETMAGICKINFOLIST)
     magick_info = GetMagickInfoList("*", &number_formats);
 
+#else
+    // IM 6.1.3 added an exception argument to GetMagickInfoList
+    GetExceptionInfo(&exception);
+    magick_info = GetMagickInfoList("*", &number_formats, &exception);
+    HANDLE_ERROR
+#endif
+
     for(x = 0; x < number_formats; x++)
     {
         rb_hash_aset(formats
@@ -353,7 +388,11 @@ Magick_set_monitor(VALUE class, VALUE monitor)
     {
         Magick_Monitor = rb_intern(MAGICK_MONITOR_CVAR);
         rb_define_class_variable(Module_Magick, MAGICK_MONITOR_CVAR, monitor);
-        ID_call = rb_intern("call");
+
+#if defined(HAVE_SETIMAGEPROGRESSMONITOR)
+        rb_warning("Magick.set_monitor is deprecated; use Image#monitor= or Image::Info#monitor= instead.");
+#endif
+
     }
 
     // If nil, turn off monitoring.
@@ -369,7 +408,7 @@ Magick_set_monitor(VALUE class, VALUE monitor)
         // available, use the 3-parm version of rb_cvar_set.
         RUBY18(rb_cvar_set(Module_Magick, Magick_Monitor, monitor, 0);)
         RUBY16(rb_cvar_set(Module_Magick, Magick_Monitor, monitor);)
-        (void) SetMonitorHandler(&monitor_handler);
+        (void) SetMonitorHandler((MonitorHandler)&monitor_handler);
     }
 
     return class;
@@ -433,13 +472,8 @@ Magick_set_log_event_mask(int argc, VALUE *argv, VALUE class)
 static VALUE
 Magick_set_log_format(VALUE class, VALUE format)
 {
-#ifdef HAVE_SETLOGFORMAT
     SetLogFormat(STRING_PTR(format));
     return class;
-#else
-    rm_not_implemented();
-    return (VALUE)0;
-#endif
 }
 
 
@@ -457,6 +491,7 @@ Init_RMagick(void)
     volatile VALUE observable;
 
     InitializeMagick("RMagick");
+    test_Magick_version();
 
     Module_Magick = rb_define_module("Magick");
 
@@ -465,6 +500,7 @@ Init_RMagick(void)
     /*-----------------------------------------------------------------------*/
 
     ID__dummy_img_      = rb_intern("_dummy_img_");
+    ID_call             = rb_intern("call");
     ID_changed          = rb_intern("changed");
     ID_cur_image        = rb_intern("cur_image");
     ID_dup              = rb_intern("dup");
@@ -521,6 +557,7 @@ Init_RMagick(void)
     rb_define_singleton_method(Class_Image, "capture", Image_capture, -1);
     rb_define_singleton_method(Class_Image, "ping", Image_ping, 1);
     rb_define_singleton_method(Class_Image, "read", Image_read, 1);
+    rb_define_singleton_method(Class_Image, "read_inline", Image_read_inline, 1);
     rb_define_singleton_method(Class_Image, "from_blob", Image_from_blob, 1);
 
     DCL_ATTR_ACCESSOR(Image, background_color)
@@ -560,27 +597,35 @@ Init_RMagick(void)
     DCL_ATTR_ACCESSOR(Image, matte_color)
     DCL_ATTR_READER(Image, mean_error_per_pixel)
     DCL_ATTR_READER(Image, mime_type)
+    DCL_ATTR_WRITER(Image, monitor)
     DCL_ATTR_ACCESSOR(Image, montage)
     DCL_ATTR_READER(Image, normalized_mean_error)
     DCL_ATTR_READER(Image, normalized_maximum_error)
     DCL_ATTR_READER(Image, number_colors)
     DCL_ATTR_ACCESSOR(Image, offset)
     DCL_ATTR_WRITER(Image, opacity)
+    DCL_ATTR_READER(Image, orientation)
     DCL_ATTR_ACCESSOR(Image, page)
+#if defined(HAVE_IMAGE_QUALITY)
+    DCL_ATTR_READER(Image, quality)
+#endif
     DCL_ATTR_READER(Image, quantum_depth)
     DCL_ATTR_ACCESSOR(Image, rendering_intent)
     DCL_ATTR_READER(Image, rows)
     DCL_ATTR_READER(Image, scene)
     DCL_ATTR_ACCESSOR(Image, start_loop)
     DCL_ATTR_ACCESSOR(Image, class_type)
+    DCL_ATTR_ACCESSOR(Image, ticks_per_second)
     DCL_ATTR_ACCESSOR(Image, tile_info)
     DCL_ATTR_READER(Image, total_colors)
     DCL_ATTR_ACCESSOR(Image, units)
+    DCL_ATTR_ACCESSOR(Image, virtual_pixel_method)
     DCL_ATTR_ACCESSOR(Image, x_resolution)
     DCL_ATTR_ACCESSOR(Image, y_resolution)
 
     rb_define_method(Class_Image, "adaptive_threshold", Image_adaptive_threshold, -1);
     rb_define_method(Class_Image, "add_noise", Image_add_noise, 1);
+    rb_define_method(Class_Image, "add_noise_channel", Image_add_noise_channel, -1);
     rb_define_method(Class_Image, "affine_transform", Image_affine_transform, 1);
     rb_define_method(Class_Image, "[]", Image_aref, 1);
     rb_define_method(Class_Image, "[]=", Image_aset, 2);
@@ -588,13 +633,16 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "bilevel_channel", Image_bilevel_channel, -1);
     rb_define_method(Class_Image, "black_threshold", Image_black_threshold, -1);
     rb_define_method(Class_Image, "blur_image", Image_blur_image, -1);
+    rb_define_method(Class_Image, "blur_channel", Image_blur_channel, -1);
     rb_define_method(Class_Image, "border", Image_border, 3);
     rb_define_method(Class_Image, "border!", Image_border_bang, 3);
     rb_define_method(Class_Image, "change_geometry", Image_change_geometry, 1);
     rb_define_method(Class_Image, "change_geometry!", Image_change_geometry, 1);
     rb_define_method(Class_Image, "changed?", Image_changed_q, 0);
     rb_define_method(Class_Image, "channel", Image_channel, 1);
-    rb_define_method(Class_Image, "channel_compare", Image_channel_compare, -1);
+    // An alias for compare_channel
+    rb_define_method(Class_Image, "channel_compare", Image_compare_channel, -1);
+    rb_define_method(Class_Image, "compare_channel", Image_compare_channel, -1);
     rb_define_method(Class_Image, "channel_depth", Image_channel_depth, -1);
     rb_define_method(Class_Image, "channel_extrema", Image_channel_extrema, -1);
     rb_define_method(Class_Image, "channel_mean", Image_channel_mean, -1);
@@ -621,6 +669,7 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "difference", Image_difference, 1);
     rb_define_method(Class_Image, "dispatch", Image_dispatch, -1);
     rb_define_method(Class_Image, "display", Image_display, 0);
+    rb_define_method(Class_Image, "distortion_channel", Image_distortion_channel, -1);
     rb_define_method(Class_Image, "_dump", Image__dump, 1);
     rb_define_method(Class_Image, "dup", Image_dup, 0);
     rb_define_method(Class_Image, "each_profile", Image_each_profile, 0);
@@ -629,7 +678,8 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "enhance", Image_enhance, 0);
     rb_define_method(Class_Image, "equalize", Image_equalize, 0);
     rb_define_method(Class_Image, "erase!", Image_erase_bang, 0);
-    rb_define_method(Class_Image, "export_pixels", Image_export_pixels, 5);
+    rb_define_method(Class_Image, "export_pixels", Image_export_pixels, -1);
+    rb_define_method(Class_Image, "export_pixels_to_str", Image_export_pixels_to_str, -1);
     rb_define_method(Class_Image, "flip", Image_flip, 0);
     rb_define_method(Class_Image, "flip!", Image_flip_bang, 0);
     rb_define_method(Class_Image, "flop", Image_flop, 0);
@@ -644,7 +694,7 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "grey?", Image_gray_q, 0);
     rb_define_method(Class_Image, "grayscale_pseudo_class", Image_grayscale_pseudo_class, -1);
     rb_define_method(Class_Image, "implode", Image_implode, -1);
-    rb_define_method(Class_Image, "import_pixels", Image_import_pixels, 6);
+    rb_define_method(Class_Image, "import_pixels", Image_import_pixels, -1);
     rb_define_method(Class_Image, "initialize_copy", Image_init_copy, 1);
     rb_define_method(Class_Image, "inspect", Image_inspect, 0);
     rb_define_method(Class_Image, "level", Image_level, -1);
@@ -662,10 +712,11 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "negate", Image_negate, -1);
     rb_define_method(Class_Image, "negate_channel", Image_negate_channel, -1);
     rb_define_method(Class_Image, "normalize", Image_normalize, 0);
+    rb_define_method(Class_Image, "normalize_channel", Image_normalize_channel, -1);
     rb_define_method(Class_Image, "oil_paint", Image_oil_paint, -1);
     rb_define_method(Class_Image, "opaque", Image_opaque, 2);
     rb_define_method(Class_Image, "opaque?", Image_opaque_q, 0);
-    rb_define_method(Class_Image, "ordered_dither", Image_ordered_dither, 0);
+    rb_define_method(Class_Image, "ordered_dither", Image_ordered_dither, -1);
     rb_define_method(Class_Image, "palette?", Image_palette_q, 0);
     rb_define_method(Class_Image, "pixel_color", Image_pixel_color, -1);
     rb_define_method(Class_Image, "posterize", Image_posterize, -1);
@@ -689,14 +740,20 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "scale", Image_scale, -1);
     rb_define_method(Class_Image, "scale!", Image_scale_bang, -1);
     rb_define_method(Class_Image, "segment", Image_segment, -1);
+    rb_define_method(Class_Image, "sepiatone", Image_sepiatone, -1);
+    rb_define_method(Class_Image, "set_channel_depth", Image_set_channel_depth, 2);
     rb_define_method(Class_Image, "shade", Image_shade, -1);
+    rb_define_method(Class_Image, "shadow", Image_shadow, -1);
     rb_define_method(Class_Image, "sharpen", Image_sharpen, -1);
+    rb_define_method(Class_Image, "sharpen_channel", Image_sharpen_channel, -1);
     rb_define_method(Class_Image, "shave", Image_shave, 2);
     rb_define_method(Class_Image, "shave!", Image_shave_bang, 2);
     rb_define_method(Class_Image, "shear", Image_shear, 2);
+    rb_define_method(Class_Image, "sigmoidal_contrast_channel", Image_sigmoidal_contrast_channel, -1);
     rb_define_method(Class_Image, "signature", Image_signature, 0);
     rb_define_method(Class_Image, "solarize", Image_solarize, -1);
     rb_define_method(Class_Image, "<=>", Image_spaceship, 1);
+    rb_define_method(Class_Image, "splice", Image_splice, -1);
     rb_define_method(Class_Image, "spread", Image_spread, -1);
     rb_define_method(Class_Image, "statistics", Image_statistics, 0);
     rb_define_method(Class_Image, "stegano", Image_stegano, 2);
@@ -714,7 +771,9 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "transparent", Image_transparent, -1);
     rb_define_method(Class_Image, "trim", Image_trim, 0);
     rb_define_method(Class_Image, "trim!", Image_trim_bang, 0);
-    rb_define_method(Class_Image, "unsharp_mask", Image_unsharp_mask, 4);
+    rb_define_method(Class_Image, "unsharp_mask", Image_unsharp_mask, -1);
+    rb_define_method(Class_Image, "unsharp_mask_channel", Image_unsharp_mask_channel, -1);
+    rb_define_method(Class_Image, "vignette", Image_vignette, -1);
     rb_define_method(Class_Image, "wave", Image_wave, -1);
     rb_define_method(Class_Image, "white_threshold", Image_white_threshold, -1);
     rb_define_method(Class_Image, "write", Image_write, 1);
@@ -729,7 +788,7 @@ Init_RMagick(void)
     rb_define_alias(Class_ImageList, "__display__", "display");
 
     // Define an alias for Array's "map" method.
-    rb_define_alias(Class_ImageList, "__map__", "map");
+    rb_define_alias(Class_ImageList, "__ary_map__", "map");
 
     rb_define_method(Class_ImageList, "animate", ImageList_animate, -1);
     rb_define_method(Class_ImageList, "append", ImageList_append, 1);
@@ -759,6 +818,7 @@ Init_RMagick(void)
 
     DCL_ATTR_WRITER(Draw, affine)
     DCL_ATTR_WRITER(Draw, align)
+	DCL_ATTR_WRITER(Draw, border_color)
     DCL_ATTR_WRITER(Draw, decorate)
     DCL_ATTR_WRITER(Draw, density)
     DCL_ATTR_WRITER(Draw, encoding)
@@ -782,6 +842,7 @@ Init_RMagick(void)
     rb_define_method(Class_Draw, "draw", Draw_draw, 1);
     rb_define_method(Class_Draw, "dup", Draw_dup, 0);
     rb_define_method(Class_Draw, "get_type_metrics", Draw_get_type_metrics, -1);
+    rb_define_method(Class_Draw, "get_multiline_type_metrics", Draw_get_multiline_type_metrics, -1);
     rb_define_method(Class_Draw, "initialize", Draw_initialize, 0);
     rb_define_method(Class_Draw, "initialize_copy", Draw_init_copy, 1);
     rb_define_method(Class_Draw, "inspect", Draw_inspect, 0);
@@ -816,6 +877,13 @@ Init_RMagick(void)
     DCL_ATTR_ACCESSOR(Pixel, blue)
     DCL_ATTR_ACCESSOR(Pixel, opacity)
 
+    // Define the CMYK attributes
+    DCL_ATTR_ACCESSOR(Pixel, cyan)
+    DCL_ATTR_ACCESSOR(Pixel, magenta)
+    DCL_ATTR_ACCESSOR(Pixel, yellow)
+    DCL_ATTR_ACCESSOR(Pixel, black)
+
+
     // Define the instance methods
     rb_define_method(Class_Pixel, "<=>", Pixel_spaceship, 1);
     rb_define_method(Class_Pixel, "===", Pixel_case_eq, 1);
@@ -878,27 +946,41 @@ Init_RMagick(void)
     rb_define_method(Class_Info, "initialize", Info_initialize, 0);
     rb_define_method(Class_Info, "freeze", rm_no_freeze, 0);
     rb_define_method(Class_Info, "define", Info_define, -1);
+    rb_define_method(Class_Info, "[]=", Info_aset, 3);
+    rb_define_method(Class_Info, "[]", Info_aref, 2);
+    rb_define_method(Class_Info, "undefine", Info_undefine, 2);
 
     DCL_ATTR_ACCESSOR(Info, antialias)
+    DCL_ATTR_ACCESSOR(Info, authenticate)
     DCL_ATTR_ACCESSOR(Info, background_color)
     DCL_ATTR_ACCESSOR(Info, border_color)
     DCL_ATTR_ACCESSOR(Info, colorspace)
+    DCL_ATTR_ACCESSOR(Info, comment)        // new in 6.0.0
     DCL_ATTR_ACCESSOR(Info, compression)
+    DCL_ATTR_ACCESSOR(Info, delay)          // new in 6.0.0
     DCL_ATTR_ACCESSOR(Info, density)
     DCL_ATTR_ACCESSOR(Info, depth)
+    DCL_ATTR_ACCESSOR(Info, dispose)        // new in 6.0.0
     DCL_ATTR_ACCESSOR(Info, dither)
     DCL_ATTR_ACCESSOR(Info, extract)    // new in 5.5.6, replaces tile
     DCL_ATTR_ACCESSOR(Info, filename)
+    DCL_ATTR_ACCESSOR(Info, fill)
     DCL_ATTR_ACCESSOR(Info, font)
     DCL_ATTR_ACCESSOR(Info, format)
     DCL_ATTR_ACCESSOR(Info, fuzz)
+    DCL_ATTR_ACCESSOR(Info, gravity)
     DCL_ATTR_ACCESSOR(Info, group)
     DCL_ATTR_ACCESSOR(Info, interlace)
+    DCL_ATTR_ACCESSOR(Info, label)          // new in 6.0.0
     DCL_ATTR_ACCESSOR(Info, matte_color)
+    DCL_ATTR_WRITER(Info, monitor)
     DCL_ATTR_ACCESSOR(Info, monochrome)
     DCL_ATTR_ACCESSOR(Info, number_scenes)  // new in 5.5.6, replaces subrange
+    DCL_ATTR_ACCESSOR(Info, orientation)    // new in 6.0.0
     DCL_ATTR_ACCESSOR(Info, page)
+    DCL_ATTR_ACCESSOR(Info, pointsize)
     DCL_ATTR_ACCESSOR(Info, quality)
+    DCL_ATTR_ACCESSOR(Info, sampling_factor)
     DCL_ATTR_ACCESSOR(Info, scene)      // new in 5.5.6, replaces subimage
     DCL_ATTR_ACCESSOR(Info, server_name)
     DCL_ATTR_ACCESSOR(Info, size)
@@ -1007,6 +1089,9 @@ Init_RMagick(void)
 #if defined(HAVE_INDEXCHANNEL)
         ENUMERATOR(IndexChannel)    // 5.5.8
 #endif
+#if defined(HAVE_GRAYCHANNEL)
+        ENUMERATOR(GrayChannel)
+#endif
 #if defined(HAVE_ALLCHANNELS)
         ENUMERATOR(AllChannels)
 #endif
@@ -1037,14 +1122,31 @@ Init_RMagick(void)
                       , rm_enum_new(Class_ColorspaceType
                       , ID2SYM(rb_intern("SRGBColorspace"))
                       , INT2FIX(sRGBColorspace)));
-#if defined(HAVE_HSLCOLORSPACE)
-        ENUMERATOR(HSLColorspace)       // 5.5.7
-#endif
-#if defined(HAVE_HWBCOLORSPACE)
-        ENUMERATOR(HWBColorspace)       // 5.5.7
-#endif
+        ENUMERATOR(HSLColorspace)
+        ENUMERATOR(HWBColorspace)
 #if defined(HAVE_HSBCOLORSPACE)
-        ENUMERATOR(HSBColorspace)       // 6.0.0
+        ENUMERATOR(HSBColorspace)           // IM 6.0.0
+#endif
+#if defined(HAVE_CINEONLOGRGBCOLORSPACE)
+        ENUMERATOR(CineonLogRGBColorspace)  // GM 1.2
+#endif
+#if defined(HAVE_LABCOLORSPACE)
+        ENUMERATOR(LABColorspace)           // GM 1.2
+#endif
+#if defined(HAVE_REC601LUMACOLORSPACE)
+        ENUMERATOR(Rec601LumaColorspace)    // GM 1.2 && IM 6.2.2
+#endif
+#if defined(HAVE_REC601YCBCRCOLORSPACE)
+        ENUMERATOR(Rec601YCbCrColorspace)   // GM 1.2 && IM 6.2.2
+#endif
+#if defined(HAVE_REC709LUMACOLORSPACE)
+        ENUMERATOR(Rec709LumaColorspace)    // GM 1.2 && IM 6.2.2
+#endif
+#if defined(HAVE_REC709YCBCRCOLORSPACE)
+        ENUMERATOR(Rec709YCbCrColorspace)   // GM 1.2 && IM 6.2.2
+#endif
+#if defined(HAVE_LOGCOLORSPACE)
+        ENUMERATOR(LogColorspace)           // IM 6.2.3
 #endif
     END_ENUM
 
@@ -1058,9 +1160,7 @@ Init_RMagick(void)
                   , ID2SYM(rb_intern("AllCompliance"))
                   , INT2FIX(SVGCompliance|X11Compliance|XPMCompliance)));
 
-#if defined(HAVE_NOCOMPLIANCE)
         ENUMERATOR(NoCompliance)
-#endif
         ENUMERATOR(SVGCompliance)
         ENUMERATOR(X11Compliance)
         ENUMERATOR(XPMCompliance)
@@ -1125,6 +1225,8 @@ Init_RMagick(void)
         ENUMERATOR(XorCompositeOp)
 
 #if defined(HAVE_COLORDODGECOMPOSITEOP)
+        ENUMERATOR(BlendCompositeOp)
+        ENUMERATOR(ColorBurnCompositeOp)
         ENUMERATOR(ColorDodgeCompositeOp)
         ENUMERATOR(ExclusionCompositeOp)
         ENUMERATOR(HardLightCompositeOp)
@@ -1140,9 +1242,13 @@ Init_RMagick(void)
         ENUMERATOR(FaxCompression)
         ENUMERATOR(Group4Compression)
         ENUMERATOR(JPEGCompression)
+#if defined(HAVE_JPEG2000COMPRESSION)
+        ENUMERATOR(JPEG2000Compression)
+#endif
         ENUMERATOR(LosslessJPEGCompression)
         ENUMERATOR(LZWCompression)
-        ENUMERATOR(RunlengthEncodedCompression)
+        ENUMERATOR(RLECompression)              // preferred
+        ENUMERATOR(RunlengthEncodedCompression) // deprecated
         ENUMERATOR(ZipCompression)
     END_ENUM
 
@@ -1154,15 +1260,13 @@ Init_RMagick(void)
         ENUMERATOR(LineThroughDecoration)
     END_ENUM
 
-#if defined(HAVE_DISPOSETYPE)
-    // DisposeType constants (5.5.1)
+    // DisposeType constants
     DEF_ENUM(DisposeType)
         ENUMERATOR(UndefinedDispose)
         ENUMERATOR(BackgroundDispose)
         ENUMERATOR(NoneDispose)
         ENUMERATOR(PreviousDispose)
     END_ENUM
-#endif
 
     DEF_ENUM(EndianType)
         ENUMERATOR(UndefinedEndian)
@@ -1192,6 +1296,13 @@ Init_RMagick(void)
 
     // GravityType constants
     DEF_ENUM(GravityType)
+#if defined(HAVE_UNDEFINEDGRAVITY)
+        ENUMERATOR(UndefinedGravity)
+#else
+        // Provide this enumerator in older (pre 6.0.0) versions of ImageMagick
+        _enum = rm_enum_new(_cls, ID2SYM(rb_intern("UndefinedGravity")), INT2FIX(0));\
+        rb_define_const(Module_Magick, "UndefinedGravity", _enum);
+#endif
         ENUMERATOR(ForgetGravity)
         ENUMERATOR(NorthWestGravity)
         ENUMERATOR(NorthGravity)
@@ -1250,6 +1361,21 @@ Init_RMagick(void)
         ENUMERATOR(PoissonNoise)
     END_ENUM
 
+#if defined(HAVE_IMAGE_ORIENTATION)
+    // Orientation constants
+    DEF_ENUM(OrientationType)
+        ENUMERATOR(UndefinedOrientation)
+        ENUMERATOR(TopLeftOrientation)
+        ENUMERATOR(TopRightOrientation)
+        ENUMERATOR(BottomRightOrientation)
+        ENUMERATOR(BottomLeftOrientation)
+        ENUMERATOR(LeftTopOrientation)
+        ENUMERATOR(RightTopOrientation)
+        ENUMERATOR(RightBottomOrientation)
+        ENUMERATOR(LeftBottomOrientation)
+    END_ENUM
+#endif
+
     // Paint method constants
     DEF_ENUM(PaintMethod)
         ENUMERATOR(PointMethod)
@@ -1324,6 +1450,22 @@ Init_RMagick(void)
         ENUMERATOR(PixelsPerCentimeterResolution)
     END_ENUM
 
+    // StorageType
+    DEF_ENUM(StorageType)
+#if defined(HAVE_UNDEFINEDGRAVITY)      // UndefinedGravity & UndefinedPixel were both introduced in IM 6.0.0
+        ENUMERATOR(UndefinedPixel)
+#endif
+        ENUMERATOR(CharPixel)
+        ENUMERATOR(DoublePixel)
+        ENUMERATOR(FloatPixel)
+        ENUMERATOR(IntegerPixel)
+        ENUMERATOR(LongPixel)
+#if defined(HAVE_QUANTUMPIXEL)
+        ENUMERATOR(QuantumPixel)
+#endif
+        ENUMERATOR(ShortPixel)
+    END_ENUM
+
     // StretchType constants
     DEF_ENUM(StretchType)
         ENUMERATOR(NormalStretch)
@@ -1346,6 +1488,18 @@ Init_RMagick(void)
         ENUMERATOR(AnyStyle)
     END_ENUM
 
+    // VirtualPixelMethod
+    DEF_ENUM(VirtualPixelMethod)
+        ENUMERATOR(UndefinedVirtualPixelMethod)
+        ENUMERATOR(EdgeVirtualPixelMethod)
+        ENUMERATOR(MirrorVirtualPixelMethod)
+        ENUMERATOR(TileVirtualPixelMethod)
+#if defined(HAVE_TRANSPARENTVIRTUALPIXELMETHOD)
+        ENUMERATOR(BackgroundVirtualPixelMethod)
+        ENUMERATOR(TransparentVirtualPixelMethod)
+#endif
+    END_ENUM
+
     // WeightType constants
     DEF_ENUM(WeightType)
         ENUMERATOR(AnyWeight)
@@ -1363,23 +1517,23 @@ Init_RMagick(void)
     // namespace. The only way to use these classes is via the Magick:: namespace.
 
     // Magick::AffineMatrix
-    Class_AffineMatrix = rb_struct_define(NULL, "sx", "rx", "ry", "sy", "tx", "ty", 0);
+    Class_AffineMatrix = rb_struct_define(NULL, "sx", "rx", "ry", "sy", "tx", "ty", NULL);
     rb_define_const(Module_Magick, "AffineMatrix", Class_AffineMatrix);
 
 #if defined(HAVE_GETIMAGESTATISTICS)
     // These classes are defined for >= GM 1.1.
 
     // Magick::Statistics
-    Class_Statistics = rb_struct_define(NULL, "red", "green", "blue", "opacity", 0);
+    Class_Statistics = rb_struct_define(NULL, "red", "green", "blue", "opacity", NULL);
     rb_define_const(Module_Magick, "Statistics", Class_Statistics);
     // Magick::ChannelStatistics
-    Class_StatisticsChannel = rb_struct_define(NULL, "max", "min", "mean", "stddev", "var", 0);
+    Class_StatisticsChannel = rb_struct_define(NULL, "max", "min", "mean", "stddev", "var", NULL);
     rb_define_const(Class_Statistics, "Channel", Class_StatisticsChannel);
 #endif
 
 
     // Magick::Primary
-    Class_Primary = rb_struct_define(NULL, "x", "y", "z", 0);
+    Class_Primary = rb_struct_define(NULL, "x", "y", "z", NULL);
     rb_define_method(Class_Primary, "to_s", PrimaryInfo_to_s, 0);
     rb_define_const(Module_Magick, "Primary", Class_Primary);
 
@@ -1389,46 +1543,104 @@ Init_RMagick(void)
                                             , "green_primary"
                                             , "blue_primary"
                                             , "white_point"
-                                            , 0);
+                                            , NULL);
     rb_define_method(Class_Chromaticity, "to_s", ChromaticityInfo_to_s, 0);
     rb_define_const(Module_Magick, "Chromaticity", Class_Chromaticity);
 
     // Magick::Color
-    Class_Color = rb_struct_define(NULL, "name", "compliance", "color", 0);
+    Class_Color = rb_struct_define(NULL, "name", "compliance", "color", NULL);
     rb_define_method(Class_Color, "to_s", Color_to_s, 0);
     rb_define_const(Module_Magick, "Color", Class_Color);
 
     // Magick::Point
-    Class_Point = rb_struct_define(NULL, "x", "y", 0);
+    Class_Point = rb_struct_define(NULL, "x", "y", NULL);
     rb_define_const(Module_Magick, "Point", Class_Point);
 
     // Magick::Rectangle
-    Class_Rectangle = rb_struct_define(NULL, "width", "height", "x", "y", 0);
+    Class_Rectangle = rb_struct_define(NULL, "width", "height", "x", "y", NULL);
     rb_define_method(Class_Rectangle, "to_s", RectangleInfo_to_s, 0);
     rb_define_const(Module_Magick, "Rectangle", Class_Rectangle);
 
     // Magick::Segment
-    Class_Segment = rb_struct_define(NULL, "x1", "y1", "x2", "y2", 0);
+    Class_Segment = rb_struct_define(NULL, "x1", "y1", "x2", "y2", NULL);
     rb_define_method(Class_Segment, "to_s", SegmentInfo_to_s, 0);
     rb_define_const(Module_Magick, "Segment", Class_Segment);
 
     // Magick::Font
     Class_Font = rb_struct_define(NULL, "name", "description",
                                       "family", "style", "stretch", "weight",
-                                      "encoding", "foundry", "format", 0);
+                                      "encoding", "foundry", "format", NULL);
     rb_define_method(Class_Font, "to_s", Font_to_s, 0);
     rb_define_const(Module_Magick, "Font", Class_Font);
 
     // Magick::TypeMetric
     Class_TypeMetric = rb_struct_define(NULL, "pixels_per_em", "ascent", "descent",
                                         "width", "height", "max_advance", "bounds",
-                                        "underline_position", "underline_thickness", 0);
+                                        "underline_position", "underline_thickness", NULL);
     rb_define_method(Class_TypeMetric, "to_s", TypeMetric_to_s, 0);
     rb_define_const(Module_Magick, "TypeMetric", Class_TypeMetric);
 
 
 }
 
+
+
+
+/*
+ *  Static:     test_Magick_version
+ *  Purpose:    Ensure the version of ImageMagick we're running with matches
+ *              the version we were compiled with.
+ *  Notes:      Bypass the test by defining the constant RMAGICK_BYPASS_VERSION_TEST
+ *              to 'true' at the top level, before requiring 'RMagick'
+*/
+static void test_Magick_version(void)
+{
+    unsigned long version_number;
+    const char *version_str;
+    const char *web_site =
+#if defined(MagickHomeURL)
+        MagickHomeURL
+#else
+        MagickWebSite
+#endif
+        ;
+    int x, n;
+    ID bypass = rb_intern("RMAGICK_BYPASS_VERSION_TEST");
+
+    if (RTEST(rb_const_defined(rb_cObject, bypass)) && RTEST(rb_const_get(rb_cObject, bypass)))
+    {
+        return;
+    }
+
+    version_str = GetMagickVersion(&version_number);
+    if (version_number != MagickLibVersion)
+    {
+        // Extract the string "ImageMagick X.Y.Z"
+        n = 0;
+        for (x = 0; version_str[x] != '\0'; x++)
+        {
+            if (version_str[x] == ' ' && ++n == 2)
+            {
+                break;
+            }
+        }
+
+        rb_raise(rb_eRuntimeError,
+            "This version of RMagick was created to run with %s %s\n"
+            "but %.*s is installed on this system. You should either\n"
+            "   1) refer to the RMagick README file to learn how to create\n"
+            "      a version of RMagick for %.*s, or\n"
+            "   2) download %s %s from %s and install it." ,
+            MagickPackageName, MagickLibVersionText, x, version_str, x, version_str,
+            MagickPackageName, MagickLibVersionText, web_site);
+    }
+
+}
+
+
+
+
+
 /*
     Static:     version_constants
     Purpose:    create Version, Magick_version, and Version_long constants.
@@ -1436,19 +1648,29 @@ Init_RMagick(void)
 static void version_constants(void)
 {
     const char *mgk_version;
-    char long_version[300];
+    volatile VALUE str;
+    char long_version[1000];
 
     mgk_version = GetMagickVersion(NULL);
-    rb_define_const(Module_Magick, "Magick_version", rb_str_new2(mgk_version));
 
-    rb_define_const(Module_Magick, "Version", rb_str_new2(PACKAGE_STRING));
+    str = rb_str_new2(mgk_version);
+    rb_obj_freeze(str);
+    rb_define_const(Module_Magick, "Magick_version", str);
+
+    str = rb_str_new2(PACKAGE_STRING);
+    rb_obj_freeze(str);
+    rb_define_const(Module_Magick, "Version", str);
+
     sprintf(long_version,
-        "This is %s ($Date: 2004/08/18 23:22:36 $) Copyright (C) 2004 by Timothy P. Hunter\n"
+        "This is %s ($Date: 2006/02/19 17:12:20 $) Copyright (C) 2006 by Timothy P. Hunter\n"
         "Built with %s\n"
         "Built for %s\n"
         "Web page: http://rmagick.rubyforge.org\n"
         "Email: rmagick@rubyforge.org\n",
         PACKAGE_STRING, mgk_version, RUBY_VERSION_STRING);
-    rb_define_const(Module_Magick, "Long_version", rb_str_new2(long_version));
+
+    str = rb_str_new2(long_version);
+    rb_obj_freeze(str);
+    rb_define_const(Module_Magick, "Long_version", str);
 
 }
